__all__ = ['build_scopus_df_from_api',
          ]

def build_scopus_df_from_api(doi_list, timeout = None, verbose = False):
    '''The function `build_scopus_df_from_api` gets, for each of the DOI 
    in the DOI list "doi_list", the hierarchical dict of the data returned by 
    the function `get_doi_json_data_from_api`. Then it builts the dataframes resulting 
    from the hierarchical dicts parsing using the function `parse_json_data_to_scopus_df`.
    Finally, it concatenates these dataframe to a single dataframe.
    
    Args:
        doi_list (list): The list of DOI (str) for the Scopus API request.
        timeout (int): The maximum waiting time in seconds for request answer.
        
    Returns:
        ((pandas.core.frame.DataFrame, pandas.core.frame.DataFrame, bool)): Tuple where first term  
        is the concatenation of the dataframes resulting from the hierarchical dicts parsing 
        for each DOI of the list "doi_list", second term is the dataframe giving the failed DOIs 
        with the reasons of their fail and third term is equal to False if the authentication 
        to the scopus database failed.
        
    Note:
        The function `get_doi_json_data_from_api` is imported from the 'api_manager' module.
        The function `parse_json_data_to_scopus_df` is imported from the 'json_parser' module.
    
    '''
    # 3rd party imports
    import pandas as pd
    
    # Local imports
    import ScopusApyJson.GLOBALS as saj_globals
    from ScopusApyJson.api_manager import get_doi_json_data_from_api
    from ScopusApyJson.json_parser import parse_json_data_to_scopus_df
    
    if not isinstance(doi_list, list): doi_list = [doi_list]
    scopus_df_list = []
    failed_num = 0
    failed_doi_df = pd.DataFrame(columns = ["DOI","Fail reason"])
    api_scopus_df = pd.DataFrame(columns = saj_globals.SELECTED_SCOPUS_COLUMNS_NAMES)
    for _, doi in enumerate(doi_list):        
        api_json_data, request_status = get_doi_json_data_from_api(doi, timeout = timeout)
        if request_status == "False":
            authy_status = False
            if verbose : print('Authentication failed: please check availability of authentication keys')
            break
        else:
            authy_status = True
            if request_status == "Empty":
                if verbose : print(f'DOI {doi} not found in Scopus database')
                failed_doi_df.loc[failed_num, "DOI"] = doi
                failed_doi_df.loc[failed_num, "Fail reason"] = "Not found"
                failed_num += 1            
            elif request_status == "Timeout":
                if verbose : print('Request timeout: please check web access')
                failed_doi_df.loc[failed_num, "DOI"] = doi
                failed_doi_df.loc[failed_num, "Fail reason"] = "Timeout"
                failed_num += 1            
            else:
                if verbose : print(f'Request successful for DOI {doi}')           
                scopus_df = parse_json_data_to_scopus_df(api_json_data)
                scopus_df_list.append(scopus_df)
                api_scopus_df = pd.concat(scopus_df_list, axis = 0)

    return (api_scopus_df, failed_doi_df, authy_status)



__all__ = ['check_not_none',
           'check_true_to_append',
           'check_true_to_set',
           'get_json_key_value',
          ]

def get_json_key_value(dic, key):
    '''Gets the value at key "key" of a hierarchical dict "dic".
    It checks the possible multiple occurences of the key.
    It returns only the first value in the case of multiple occurences of the key.
    
    Args:
        dic (dict): The hierarchical dict.
        key (str): The key which value is searched in the dict "dic".
        
    Returns:
        (str, list or dict): The first value at key "key" in the hierarchical 
        dict "dic" whatever the number of occurences of the key, 
        or None if the dict is empty or if the key is not present.
    
    '''
    if dic:
        hit = []
        stack = list(zip(dic.keys(), dic.values()))
        while stack :
            k,v = stack.pop()
            if k == key:
                hit.append(v)
            if isinstance(v, dict):
                stack.extend(list(zip(v.keys(), v.values()))) 
        if len(hit): 
            return hit[0]
        else: return None
    else: return None

def check_not_none(value):
    '''Checks if the parameter "value" is not None or not equal to "None".
    
    Args:
        value (str): The parameter to check.
        
    Returns:
        (bool): True if value is not None or not equal to "None" (default: False).
        
    '''
    status = False
    if value and value != "None": 
        status = True 
    return status
        
    
def check_true_to_append(dic, key, items_list):
    '''Appends to the list "items_list", the value at key "key" of a hierarchical dict "dic" 
    in case the return of the function `check_not_none` is True. 
    
    Args:
        dic (dict): The hierarchical dict.
        key (str): The key which value is searched in the dict "dic".
        items_list (list): The list to append.
        
    Returns:
        (list): The list appended with the value at key "key" of a hierarchical 
        dict "dic" (default: unchanged list).
        
    '''
    value = get_json_key_value(dic, key)
    if check_not_none(value): items_list.append(value)
    return items_list


def check_true_to_set(dic1, key1, dic2, key2):
    '''Sets the value at key "key2" of a dict "dic2" 
    to the value "value" returned by the function `get_json_key_value` 
    applied to the hierarchical dict "dic1" and key "key1" 
    in case the return of the function `check_not_none` 
    applied to value "value" is True. 
    
    Args:
        dic1 (dict): The hierarchical dict.
        key1 (str): The key which value is searched in the dict "dic1".
        dic2 (dict): The dict to be updated.
        key2 (str): The key at which the dict "dic2" will be updated.
        
    Returns:
        (dict): The dict "dic2" updated at key "key2" with the value at key "key1" of the hierarchical 
        dict "dic1" (default: unchanged dict "dic2").
        
    '''    
    value = get_json_key_value(dic1, key1)
    if check_not_none(value): dic2[key2] = value
    return dic2
